class ElementContainer {
    constructor(shadowHost, shadowRoot) {
        this.shadowHost = shadowHost;
        this.shadowRoot = shadowRoot;
        this.isOpen = false;
    }
    insertSelf() {
        if (!this.isOpen) {
            document.body.appendChild(this.shadowHost);
            this.getModalElement().classList.add('animate');
            this.isOpen = true;
        }
    }
    removeSelf() {
        if (this.isOpen) {
            document.body.removeChild(this.shadowHost);
            this.isOpen = false;
        }
    }
    getModalElement() {
        return this.shadowRoot.querySelector('.modal-dialog');
    }
    getContentElement() {
        return this.shadowRoot.querySelector('.modal-body');
    }
    addCloseButtonListener(onClose) {
        // Remember there are 2 close buttons
        this.shadowRoot.querySelectorAll('*[closeButton]')
            .forEach(item => item.addEventListener('click', onClose));
    }
    addConfirmButtonListener(onConfirm) {
        this.shadowRoot.querySelector('.btn.btn-primary[confirmButton]')
            .addEventListener('click', onConfirm);
    }
}

const templateHTML = `

<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD" crossorigin="anonymous">

<style>
.modal {
    --docsearch-text-color: #1c1e21;
    --docsearch-icon-stroke-width: 1.4;
    --docsearch-highlight-color: var(--docsearch-primary-color);
    --docsearch-muted-color: #969faf;
    --docsearch-container-background: rgba(101,108,133,0.8);
    --docsearch-modal-width: 560px;
    --docsearch-modal-height: 600px;
    --docsearch-modal-background: #f5f6f7;
    --docsearch-modal-shadow: inset 1px 1px 0 0 hsla(0,0%,100%,0.5),0 3px 8px 0 #555a64;
    --docsearch-searchbox-height: 56px;
    --docsearch-searchbox-background: #ebedf0;
    --docsearch-searchbox-focus-background: #fff;
    --docsearch-searchbox-shadow: inset 0 0 0 2px var(--docsearch-primary-color);
    --docsearch-hit-height: 56px;
    --docsearch-hit-color: #444950;
    --docsearch-hit-active-color: #fff;
    --docsearch-hit-background: #fff;
    --docsearch-hit-shadow: 0 1px 3px 0 #d4d9e1;
    --docsearch-key-gradient: linear-gradient(-225deg,#d5dbe4,#f8f8f8);
    --docsearch-key-shadow: inset 0 -2px 0 0 #cdcde6,inset 0 0 1px 1px #fff,0 1px 2px 1px rgba(30,35,90,0.4);
    --docsearch-footer-background: #fff;
    --docsearch-footer-shadow: 0 -1px 0 0 #e0e3e8,0 -3px 6px 0 rgba(69,98,155,0.12);
    --docsearch-spacing: 10px;
    --docsearch-footer-height: 40px;
    --bs-blue: #0d6efd;
    --bs-indigo: #6610f2;
    --bs-purple: #6f42c1;
    --bs-pink: #d63384;
    --bs-red: #dc3545;
    --bs-orange: #fd7e14;
    --bs-yellow: #ffc107;
    --bs-green: #198754;
    --bs-teal: #20c997;
    --bs-cyan: #0dcaf0;
    --bs-black: #000;
    --bs-white: #fff;
    --bs-gray: #6c757d;
    --bs-gray-dark: #343a40;
    --bs-gray-100: #f8f9fa;
    --bs-gray-200: #e9ecef;
    --bs-gray-300: #dee2e6;
    --bs-gray-400: #ced4da;
    --bs-gray-500: #adb5bd;
    --bs-gray-600: #6c757d;
    --bs-gray-700: #495057;
    --bs-gray-800: #343a40;
    --bs-gray-900: #212529;
    --bs-primary: #0d6efd;
    --bs-secondary: #6c757d;
    --bs-success: #198754;
    --bs-info: #0dcaf0;
    --bs-warning: #ffc107;
    --bs-danger: #dc3545;
    --bs-light: #f8f9fa;
    --bs-dark: #212529;
    --bs-primary-rgb: 13,110,253;
    --bs-secondary-rgb: 108,117,125;
    --bs-success-rgb: 25,135,84;
    --bs-info-rgb: 13,202,240;
    --bs-warning-rgb: 255,193,7;
    --bs-danger-rgb: 220,53,69;
    --bs-light-rgb: 248,249,250;
    --bs-dark-rgb: 33,37,41;
    --bs-white-rgb: 255,255,255;
    --bs-black-rgb: 0,0,0;
    --bs-body-color-rgb: 33,37,41;
    --bs-body-bg-rgb: 255,255,255;
    --bs-font-sans-serif: system-ui,-apple-system,"Segoe UI",Roboto,"Helvetica Neue","Noto Sans","Liberation Sans",Arial,sans-serif,"Apple Color Emoji","Segoe UI Emoji","Segoe UI Symbol","Noto Color Emoji";
    --bs-font-monospace: SFMono-Regular,Menlo,Monaco,Consolas,"Liberation Mono","Courier New",monospace;
    --bs-gradient: linear-gradient(180deg, rgba(255, 255, 255, 0.15), rgba(255, 255, 255, 0));
    --bs-body-font-family: var(--bs-font-sans-serif);
    --bs-body-font-size: 1rem;
    --bs-body-font-weight: 400;
    --bs-body-line-height: 1.5;
    --bs-body-color: #212529;
    --bs-body-bg: #fff;
    --bs-border-width: 1px;
    --bs-border-style: solid;
    --bs-border-color: #dee2e6;
    --bs-border-color-translucent: rgba(0, 0, 0, 0.175);
    --bs-border-radius: 0.375rem;
    --bs-border-radius-sm: 0.25rem;
    --bs-border-radius-lg: 0.5rem;
    --bs-border-radius-xl: 1rem;
    --bs-border-radius-2xl: 2rem;
    --bs-border-radius-pill: 50rem;
    --bs-heading-color: ;
    --bs-link-color: #0d6efd;
    --bs-link-hover-color: #0a58ca;
    --bs-code-color: #d63384;
    --bs-highlight-bg: #fff3cd;
    --bd-purple: #4c0bce;
    --bd-violet: #712cf9;
    --bd-accent: #ffe484;
    --bd-violet-rgb: 112.520718,44.062154,249.437846;
    --bd-accent-rgb: 255,228,132;
    --bd-pink-rgb: 214,51,132;
    --bd-teal-rgb: 32,201,151;
    --docsearch-primary-color: var(--bd-violet);
    --docsearch-logo-color: var(--bd-violet);
    --base00: #fff;
    --base01: #f5f5f5;
    --base02: #c8c8fa;
    --base03: #565c64;
    --base04: #030303;
    --base05: #333;
    --base06: #fff;
    --base07: #9a6700;
    --base08: #bc4c00;
    --base09: #087990;
    --base0A: #795da3;
    --base0B: #183691;
    --base0C: #183691;
    --base0D: #795da3;
    --base0E: #a71d5d;
    --base0F: #333;
    font-family: var(--bs-body-font-family);
    font-size: var(--bs-body-font-size);
    font-weight: var(--bs-body-font-weight);
    line-height: var(--bs-body-line-height);
    color: var(--bs-body-color);
    text-align: var(--bs-body-text-align);
    -webkit-text-size-adjust: 100%;
    -webkit-tap-highlight-color: transparent;
    --bs-gutter-y: 0;
    --bs-gutter-x: 3rem;
    box-sizing: border-box;
    transition: opacity .15s linear;
    --bs-modal-zindex: 1055;
    --bs-modal-width: 500px;
    --bs-modal-padding: 1rem;
    --bs-modal-color: ;
    --bs-modal-bg: #fff;
    --bs-modal-border-color: var(--bs-border-color-translucent);
    --bs-modal-border-width: 1px;
    --bs-modal-border-radius: 0.5rem;
    --bs-modal-inner-border-radius: calc(0.5rem - 1px);
    --bs-modal-header-padding-x: 1rem;
    --bs-modal-header-padding-y: 1rem;
    --bs-modal-header-padding: 1rem 1rem;
    --bs-modal-header-border-color: var(--bs-border-color);
    --bs-modal-header-border-width: 1px;
    --bs-modal-title-line-height: 1.5;
    --bs-modal-footer-gap: 0.5rem;
    --bs-modal-footer-bg: ;
    --bs-modal-footer-border-color: var(--bs-border-color);
    --bs-modal-footer-border-width: 1px;
    position: fixed;
    top: 0;
    left: 0;
    z-index: var(--bs-modal-zindex);
    width: 100%;
    height: 100%;
    overflow-x: hidden;
    overflow-y: auto;
    outline: 0;
    --bs-modal-margin: 1.75rem;
    --bs-modal-box-shadow: 0 0.5rem 1rem rgba(0, 0, 0, 0.15);
    display: block;
}

.animate.animate.animate {
    animation: entryAnimation 350ms ease-in-out 1;
}

@keyframes entryAnimation {
  0% {
    opacity: 0;
  }

  100% {
    opacity: 1;
  }
}
</style>

<div class="modal fade show" tabindex="-1" style="display: block; background-color: RGBA(0, 0, 0, 0.5);">
    <div class="modal-dialog modal-xl">
        <div class="modal-content">
        
            <div class="modal-header">
                <h5 class="modal-title">$TITLE</h5>
                <button type="button" class="btn-close" closeButton></button>
            </div>
            
            <div class="modal-body">
                $CONTENT
            </div>
            
            <div class="modal-footer">
                <button type="button" class="btn btn-secondary" closeButton>Close</button>
                <button type="button" class="btn btn-primary" confirmButton>Confirm</button>
            </div>
            
        </div>
    </div>
</div>

`;

class ElementContainerFactory {
    constructor(config) {
        this.config = config;
    }
    createContainer() {
        const root = document.createElement('div');
        ElementContainerFactory.setRootStyles(root);
        const shadowRoot = root.attachShadow({ mode: "closed" });
        this.mountTemplate(shadowRoot);
        const container = new ElementContainer(root, shadowRoot);
        if (this.config.content instanceof HTMLElement) {
            container.getContentElement().appendChild(this.config.content);
        }
        return container;
    }
    static setRootStyles(root) {
        root.style.all = 'initial';
    }
    mountTemplate(shadowRoot) {
        let html = templateHTML.replace('$TITLE', this.config.header);
        if (typeof this.config.content === 'string') {
            html = html.replace('$CONTENT', this.config.content);
        }
        else {
            html = html.replace('$CONTENT', '');
        }
        shadowRoot.innerHTML = html;
    }
}

/**
 * This class listens to the escape key and closes this.host if esc is pressed.
 */
class EscListener {
    constructor(host) {
        this.host = host;
        this.onKeyPress = (e) => {
            if (e.key === 'Escape') {
                this.host.close();
            }
        };
    }
    engage() {
        document.addEventListener('keyup', this.onKeyPress);
    }
    disengage() {
        document.removeEventListener('keyup', this.onKeyPress);
    }
}

/**
 * This class specialises in listening to ctrl+enter events
 */
class CtrlEnterListener {
    constructor(callback) {
        this.callback = callback;
        this.rawListener = (e) => {
            if ((e.ctrlKey || e.metaKey) && e.key === 'Enter') {
                this.callback();
            }
        };
    }
    engage() {
        document.addEventListener('keypress', this.rawListener);
    }
    disengage() {
        document.removeEventListener('keypress', this.rawListener);
    }
}

class ObjectOrientedBootstrapModal {
    constructor(config) {
        this.escListener = new EscListener(this);
        this.onConfirm = null;
        this.rawOnConfirmListener = () => {
            if (this.onConfirm) {
                this.onConfirm();
            }
        };
        this.container = new ElementContainerFactory(config).createContainer();
        if (config.onConfirm) {
            this.onConfirm = config.onConfirm;
        }
        this.container.addConfirmButtonListener(this.rawOnConfirmListener);
        this.ctrlEnterListener = new CtrlEnterListener(this.rawOnConfirmListener);
        this.container.addCloseButtonListener(() => this.close());
    }
    open() {
        this.container.insertSelf();
        this.escListener.engage();
        this.ctrlEnterListener.engage();
    }
    close() {
        this.container.removeSelf();
        this.escListener.disengage();
        this.ctrlEnterListener.disengage();
    }
    get content() {
        return this.container.getContentElement();
    }
}

export { ObjectOrientedBootstrapModal };
